NDAYS=10
INDIR="/nird/home/laurentb/Maxime/SIDFEx/SIDFEx_inputs"
OUTDIR="/cluster/projects/nn2993k/SIDFEx_outputs"
MOTUCLIENTDIR="/nird/home/laurentb/Maxime/GODAE/motu-client-python"
LOGIN_CMEMS="cmems_username"
PASSWD_CMEMS="cmems_password"

# import modules
import numpy as np
import numpy.ma as ma
import scipy as sp
from netCDF4 import Dataset
from scipy.interpolate import RectBivariateSpline
from scipy import stats
from math import sqrt, floor
from datetime import date, time, datetime, timedelta
import calendar, csv, os, imp, subprocess, sys
from time import sleep

BuoyID=str(sys.argv[1])
#datestart=str(sys.argv[2])
#dateend=str(sys.argv[3])
OUTDIR=OUTDIR+"/"+BuoyID
from ftplib import FTP
from functools import reduce
# add directory for sources
split=INDIR.split("/")
sys.path.append("/"+"/".join(split[1:len(split)-1]))
from proj_stereo import *
from offset import *
from rk2 import *
from plot_track import *

def daterange(start_date, end_date):
    for n in range(int ((end_date - start_date).days) + 1):
        yield start_date + timedelta(n)

def extract_date(filename):
    YEAR=filename.split('_')[3].split('-')[0]
    DAY=str(int(float(filename.split('_')[3].split('-')[1])))
    if len(DAY)==1:
      DAY="00"+DAY
    if len(DAY)==2:
      DAY="0"+DAY
    date=datetime.strptime(YEAR+DAY, '%Y%j')
    date=date.strftime("%Y%m%d")
    return date

# collecting new buoy data
file_data=INDIR+"/"+BuoyID+".dat"
bashCommand="wget http://iabp.apl.washington.edu/WebData/"+BuoyID+".dat -O "+file_data 
process = subprocess.Popen(bashCommand.split())#,stdin=subprocess.PIPE, stdout=subprocess.PIPE)
process.communicate()

# reading the data
file_data=INDIR+"/"+BuoyID+".dat"
buoy=np.genfromtxt(open(file_data, "rb"), delimiter='     ')
buoy=np.delete(buoy,0,axis=0)
mask=(buoy[:,4]-np.floor(buoy[:,4]))==0
buoy_00=buoy[mask,]
ldate_year=buoy_00[:,1]
ldate_day=buoy_00[:,5]
ldate=np.empty(len(ldate_year),dtype=object)
for i in range(0, len(ldate_year)):
  ldate[i]=(datetime(int(ldate_year[i]), 1, 1)+timedelta(days=int(ldate_day[i])-1)).strftime("%Y%m%d")

# checking which dates are not yet generated
lfiles=os.listdir(OUTDIR)
lfiles=[name for name in lfiles if name.split('-')[0]!="track"]
ldate2=np.unique(map(extract_date,lfiles))

# selecting the dates to run
set1 = set(ldate)
set2 = set(ldate2)
ldate=list(set1.difference(set2))
ldate=np.sort(ldate)

# check if the CMEMS data are available for all the dates
ldate=[date_ for date_ in ldate if (date.today()-datetime.strptime(date_, '%Y%m%d').date()).days>=0]
ldate_year=[(datetime.strptime(date_, '%Y%m%d').strftime("%Y")) for date_ in ldate]
ldate_day=[(datetime.strptime(date_, '%Y%m%d').strftime("%j")) for date_ in ldate]

# loop on the dates
for i in range(0, len(ldate)):
  print(ldate[i])
  start_date=datetime.strptime(ldate[i], '%Y%m%d')
  end_date=(datetime.strptime(ldate[i], '%Y%m%d')+timedelta(days=NDAYS))
  # import TOPAZ outputs from CMEMS
  bashCommand="ncrcat -O "
  for single_date in daterange(start_date, end_date):
    date=single_date.strftime("%Y%m%d")
    my_TOPAZ_file="/cluster/projects/nn2993k/export_TOPAZ/ARCTIC_ANALYSIS_FORECAST_PHYS_002_001_a_"+date+"_v2.nc"
    if os.path.exists(my_TOPAZ_file)==False:
      date2=single_date.strftime("%Y-%m-%d")
      date3=(single_date+timedelta(days=0)).strftime("%Y-%m-%d")
      filetopaz="ARCTIC_ANALYSIS_FORECAST_PHYS_002_001_a_"+date+"_v2.nc"
      topazdir="/cluster/projects/nn2993k/export_TOPAZ"
      bashCommand2 = "python " + MOTUCLIENTDIR + "/motu-client.py -u " + LOGIN_CMEMS + " -p " + PASSWD_CMEMS + " -m http://cmems.met.no/miscas/cgi-bin/fiMisSubsetter -s ARC-METNO-ARC-TOPAZ4_2_PHYS-FOR-TDS -d dataset-topaz4-arc-myoceanv2-be -x -180 -X 180 -y 34.69 -Y 90 -t "+date2+"T00:00:00 -T "+date3+"T00:00:00 -z 5 -Z 3000.0001 -v time  -v latitude -v longitude -v uice -v vice -o " + topazdir + " -f " + filetopaz
      process = subprocess.Popen(bashCommand2.split())#,stdin=subprocess.PIPE, stdout=subprocess.PIPE)
      process.communicate()
    bashCommand=bashCommand+" "+my_TOPAZ_file
  bashCommand=bashCommand+" -o /cluster/projects/nn2993k/export_TOPAZ/ARCTIC_ANALYSIS_FORECAST_PHYS_002_001_a_"+ldate[i]+"_tmp.nc"
  process = subprocess.Popen(bashCommand.split())#,stdin=subprocess.PIPE, stdout=subprocess.PIPE)
  process.communicate()
  topaz = Dataset("/cluster/projects/nn2993k/export_TOPAZ/ARCTIC_ANALYSIS_FORECAST_PHYS_002_001_a_"+ldate[i]+"_tmp.nc", mode='r')
  lons = topaz.variables['longitude'][:]
  lats = topaz.variables['latitude'][:]
  u=topaz.variables['uice'][:,:,:]
  u_units=topaz.variables['uice'].units
  v=topaz.variables['vice'][:,:,:]
  v_units=topaz.variables['vice'].units
  u, v = np.nan_to_num(u), np.nan_to_num(v)

  index_true1=np.where(buoy_00[:,4]==int(ldate_day[i]))
  index_true2=np.where(buoy_00[:,1]==int(ldate_year[i]))
  index_true=np.intersect1d(index_true1,index_true2)
  init_lon=buoy_00[index_true[0],7]
  init_lat=buoy_00[index_true[0],6]
  # projection of polar grid coordinates on regular grid
  x_,y_ = polar_stereographic(lons,lats,'ll2xy')
  regx=np.sort(np.unique(x_)) 
  regy=np.sort(np.unique(y_))
  x_=np.reshape(x_,(len(regy),len(regx)))
  y_=np.reshape(y_,(len(regy),len(regx)))
 
  #--- 1st METHOD : push-the-stone ---# 
  true_lon=np.empty(NDAYS,dtype='float64')
  true_lat=np.empty(NDAYS,dtype='float64')
  new_lon_M1=np.empty(NDAYS,dtype='float64')
  new_lat_M1=np.empty(NDAYS,dtype='float64')
  lyear=np.empty(NDAYS,dtype='float64')
  lday=np.empty(NDAYS,dtype='float64')
  new_lon_M1[0]=init_lon
  new_lat_M1[0]=init_lat
  liste_j=range(0, NDAYS-1)
  lj=0
  keepTrack=True
  while lj<len(liste_j) and keepTrack==True:
    j=liste_j[lj]
    # get the true position of the buoy (if available)
    lyear[j]=(datetime.strptime(ldate[i], '%Y%m%d')+timedelta(days=j)).year
    lday[j]=(datetime.strptime(ldate[i], '%Y%m%d')+timedelta(days=j)).timetuple().tm_yday
    index_true1=np.where(buoy_00[:,4]==lday[j])
    index_true2=np.where(buoy_00[:,1]==lyear[j])
    index_true=np.intersect1d(index_true1,index_true2)
    if len(index_true)!=0:
      true_lon[j]=buoy_00[index_true[0],7]
      true_lat[j]=buoy_00[index_true[0],6]
    else:
      true_lon[j]=np.nan
      true_lat[j]=np.nan
    # projection of init buoy coordinates on regular grid, interpolation of u and v, computation of the offsets
    x,y=polar_stereographic(new_lon_M1[j],new_lat_M1[j],'ll2xy')
    if np.isnan(x) or np.isnan(y): 
      keepTrack=False
    else:
      f2d = RectBivariateSpline(regy,regx,u[j,:,:], kx=1,ky=1)
      ux = f2d.ev(y[0],x[0])
      f2d = RectBivariateSpline(regy,regx,v[j,:,:], kx=1,ky=1)
      uy = f2d.ev(y[0],x[0])
      xjp1, yjp1 = offset(x,y,ux,uy,86400)
      new_lon_M1[j+1], new_lat_M1[j+1] = polar_stereographic(xjp1,yjp1,'xy2ll') 
    lj=lj+1
  if keepTrack==True:
    # get the final position of the buoy (if available)
    lyear[NDAYS-1]=(datetime.strptime(ldate[i], '%Y%m%d')+timedelta(days=NDAYS-1)).year
    lday[NDAYS-1]=(datetime.strptime(ldate[i], '%Y%m%d')+timedelta(days=NDAYS-1)).timetuple().tm_yday
    index_true1=np.where(buoy_00[:,4]==lday[NDAYS-1])
    index_true2=np.where(buoy_00[:,1]==lyear[NDAYS-1])
    index_true=np.intersect1d(index_true1,index_true2)
    if len(index_true)!=0:
      true_lon[NDAYS-1]=buoy_00[index_true[0],7]
      true_lat[NDAYS-1]=buoy_00[index_true[0],6]
    else:
      true_lon[NDAYS-1]=np.nan
      true_lat[NDAYS-1]=np.nan
    # export track M1 to ASCII format
    GroupID="xxxxx001"
    MethodID="PB"  # Pousse-Bouchon = Push the cork
    Inityear=str(int(ldate_year[i]))
    Initdayofyear=str(ldate_day[i])
    Initlat=str(init_lat)
    Initlon=str(init_lon)
    Ensmemnum="001"
    header="GroupID: "+GroupID+ "\n"+"MethodID: "+MethodID+ "\n"+"TargetID: "+BuoyID+ "\n"+"InitYear: "+Inityear+ "\n"+"InitDayOfYear: "+Initdayofyear+ "\n"+"InitLat: "+Initlat+ "\n"+"InitLon: "+Initlon+ "\n"+"EnsMemNum: "+Ensmemnum+ "\n"+"### end of header"+ "\n"+"Year"+"\t"+"DayOfYear"+"\t"+"Lat"+"\t"+"Lon"
    file_out=OUTDIR+"/"+GroupID+"_"+MethodID+"_"+BuoyID+"_"+Inityear+"-"+Initdayofyear+"_"+Ensmemnum+".txt"
    track_M1=np.zeros([len(new_lon_M1)-1,4],dtype=object)
    track_M1[:,0]=np.delete(lyear,0)
    track_M1[:,1]=np.delete(lday,0)
    track_M1[:,2]=np.delete(new_lat_M1,0)
    track_M1[:,3]=np.delete(new_lon_M1,0)
    np.savetxt(file_out,track_M1,fmt="%10d\t%10.5f\t%10.5f\t%10.5f",header=header,comments="") 

    #--- 2nd METHOD : Second-order Runge-Kutta (RK2) ---#
    new_lon_M2=np.empty(NDAYS,dtype='float64')
    new_lat_M2=np.empty(NDAYS,dtype='float64')
    new_lon_M2[0]=init_lon
    new_lat_M2[0]=init_lat
    delt=86400/4. # delt is the integration step (6 hours here)
    for j in range(0, NDAYS-1):
      x,y=polar_stereographic(new_lon_M2[j],new_lat_M2[j],'ll2xy')
      new_lon_M2[j+1], new_lat_M2[j+1] = rk2(u[[j,j+1],:,:],v[[j,j+1],:,:],x,y,x_,y_,delt)
      new_lon_M2[j+1], new_lat_M2[j+1] = polar_stereographic(new_lon_M2[j+1], new_lat_M2[j+1],'xy2ll')
    # export track M2 to ASCII format
    GroupID="xxxxx001"
    MethodID="RK2"
    Inityear=str(int(ldate_year[i]))
    Initdayofyear=str(ldate_day[i])
    Initlat=str(init_lat)
    Initlon=str(init_lon)
    Ensmemnum="001"
    header="GroupID: "+GroupID+ "\n"+"MethodID: "+MethodID+ "\n"+"TargetID: "+BuoyID+ "\n"+"InitYear: "+Inityear+ "\n"+"InitDayOfYear: "+Initdayofyear+ "\n"+"InitLat: "+Initlat+ "\n"+"InitLon: "+Initlon+ "\n"+"EnsMemNum: "+Ensmemnum+ "\n"+"### end of header"+ "\n"+"Year"+"\t"+"DayOfYear"+"\t"+"Lat"+"\t"+"Lon"
    file_out=OUTDIR+"/"+GroupID+"_"+MethodID+"_"+BuoyID+"_"+Inityear+"-"+Initdayofyear+"_"+Ensmemnum+".txt"
    file_out_short=GroupID+"_"+MethodID+"_"+BuoyID+"_"+Inityear+"-"+Initdayofyear+"_"+Ensmemnum+".txt"
    track_M2=np.zeros([len(new_lon_M2)-1,4],dtype=object)
    track_M2[:,0]=np.delete(lyear,0)
    track_M2[:,1]=np.delete(lday,0)
    track_M2[:,2]=np.delete(new_lat_M2,0)
    track_M2[:,3]=np.delete(new_lon_M2,0)
    np.savetxt(file_out,track_M2,fmt="%10d\t%10.5f\t%10.5f\t%10.5f",header=header,comments="") 
    # Modify according to your own SIDFEx credentials (sent to you by email from SIDFEx)
    bashCommand="#!/bin/sh -l \n curl -XPUT \"https://swift.dkrz.de/v1/dkrz_0262ea1f00e34439850f3f1d71817205/SIDFEx/incoming/xxxxx001/"+file_out_short+"?temp_url_sig=xxxxx&temp_url_expires=xxxxx&temp_url_prefix=incoming/xxxxx001\" --data-binary @/cluster/projects/nn2993k/SIDFEx_outputs/"+BuoyID+"/"+file_out_short
    text_file = open("/cluster/projects/nn2993k/SIDFEx_outputs/"+BuoyID+"_"+ldate[i]+".sh", "w")
    text_file.write(bashCommand)
    text_file.close()
    bashCommand = "chmod +x /cluster/projects/nn2993k/SIDFEx_outputs/"+BuoyID+"_"+ldate[i]+".sh"
    subprocess.call(bashCommand.split())
    bashCommand = "/cluster/projects/nn2993k/SIDFEx_outputs/"+BuoyID+"_"+ldate[i]+".sh"
    subprocess.call(bashCommand)
    bashCommand = "rm -rf /cluster/projects/nn2993k/SIDFEx_outputs/"+BuoyID+"_"+ldate[i]+".sh"
    subprocess.call(bashCommand.split())

   # plot track
    file_out=OUTDIR+"/track-buoy-" + BuoyID + "-" + ldate[i] + ".pdf"
    plot_track(BuoyID,ldate[i],file_out,lons,lats,new_lon_M1,new_lat_M1,new_lon_M2,new_lat_M2,true_lon,true_lat)

  # closing TOPAZ file and remove netcdf from OUTDIR
  topaz.close()
  bashCommand = "rm -rf /cluster/projects/nn2993k/export_TOPAZ/ARCTIC_ANALYSIS_FORECAST_PHYS_002_001_a_"+ldate[i]+"_tmp.nc"
  process = subprocess.Popen(bashCommand.split())#,stdin=subprocess.PIPE, stdout=subprocess.PIPE)
  process.communicate()

