import numpy as np
import numpy.ma as ma
import scipy as sp

def polar_stereographic(coord_x,coord_y,direction):
  Re=6378.273     # Reference radius of earth [km]
  CDR=57.29577951 # radian to degree conv factor
  # polar stereographic specific parameters
  polar_stereographic_central_lon =-45
  polar_stereographic_central_lat = 90
  proj_conv_fac = 1.
  # conversion from XY (regular) to polar-stereo
  if(direction=="xy2ll"):
    x2=coord_x.flatten()/proj_conv_fac # proj_conv_fac to get x/y in km
    y2=coord_y.flatten()/proj_conv_fac # proj_conv_fac to get x/y in km	
    rho=np.sqrt(x2**2 +y2**2)
    c=2*np.arctan(rho/(2*Re))
    lon=polar_stereographic_central_lon + np.arctan2(x2*np.sin(c),rho*np.cos(polar_stereographic_central_lat/CDR)*np.cos(c) - y2*  np.sin(polar_stereographic_central_lat/CDR)*np.sin(c))*CDR
    lat=np.empty(len(lon),dtype=object)
    for i in range(0,len(c)):
      if (c[i]==0.0):   
        lat[i]=np.arcsin(np.cos(c[i])*np.sin(polar_stereographic_central_lat/CDR))*CDR
      else:
        lat[i]=np.arcsin(np.cos(c[i])*np.sin(polar_stereographic_central_lat/CDR) + y2[i]*np.sin(c[i])*np.cos(polar_stereographic_central_lat/CDR)/rho[i])*CDR
    coord_x_reproj=lon
    coord_y_reproj=lat
  if(direction=="ll2xy"):
    # conversion from polar-stereo to XY (regular)
    k=2*Re/(1.+np.sin(polar_stereographic_central_lat/CDR)*np.sin(coord_y.flatten()/CDR) +np.cos(polar_stereographic_central_lat/CDR)*np.cos(coord_y.flatten()/CDR)*np.cos((coord_x.flatten()-polar_stereographic_central_lon)/CDR))
    y= k*(np.cos(polar_stereographic_central_lat/CDR)*np.sin(coord_y.flatten()/CDR) - np.sin(polar_stereographic_central_lat/CDR)*np.cos(coord_y.flatten()/CDR)*np.cos((coord_x.flatten()-polar_stereographic_central_lon)/CDR))*proj_conv_fac
    x= k*(np.cos(coord_y.flatten()/CDR)*np.sin((coord_x.flatten()-polar_stereographic_central_lon)/CDR))*proj_conv_fac
    x=x.round(2)
    y=y.round(2)
    coord_x_reproj=x
    coord_y_reproj=y
  return coord_x_reproj,coord_y_reproj;
